
class LogHelper {

    # 写入日志
    static [void] writeLog([string]$message, [System.Exception]$exception = $null) {
        
        if ($exception) {
            Write-Log -Level ERROR -Message $message
            Write-Log -Level ERROR -Message "$($exception)"
        }
        else {
            Write-Log -Level INFO -Message $message
        }
    }

    static [void] startLog([string] $logName = 'log') {
        [LogHelper]::startLog($logName, 'DEBUG')
    }

    # 启用日志
    static [void] startLog([string] $logName = 'log', $level) {

        $module = Get-Module -Name logging

        if ($module) {
            return
        }

        $filePath = "$($PSScriptRoot)\$($logName)_%{+%Y%m%d}.log"

        # Install-Module Logging
        Import-Module Logging

        Set-LoggingDefaultLevel -Level $level
        Add-LoggingTarget -Name Console
        Add-LoggingTarget -Name File -Configuration @{Path = $filePath; Encoding = 'UTF8' }
    }

    # 停用日志
    static [void] endLog() { Wait-Logging }
}

class RdcFile : System.IDisposable {

    [string] getRdcModulePath() {

        $rdcmanPath = Get-Process RDCMan -ErrorAction SilentlyContinue | ForEach-Object path | Select-Object -First 1
        if ($rdcmanPath) {
            return $rdcmanPath;
        }

        # 猜一个路径
        return "D:\Apps\SysinternalsSuite\RDCMan.exe"    
    }

    [void] loadModule() {

        if ($(Get-Module | Where-Object { $_ -match '\bRDCMan\b' } )) {
            return
        }

        # 使用 SysinternalsSuite 的 RDCMan 模块中的函数进行解密 
        $rdcmanPath = $this.getRdcModulePath();

        if (-not $(Test-Path $rdcmanPath)) {
            throw [System.IO.FileNotFoundException]::new("找不到模块 $($rdcmanPath)")
        }

        Import-Module $rdcmanPath
    }

    RdcFile() {
        $this.loadModule()
    }

    [void] Dispose() {

        # 释放资源
        # Remove-Module RDCMan
    }

    # 密码解码 
    [string] decodePassword($pass) {
    
        $EncryptionSettings = New-Object -TypeName RdcMan.EncryptionSettings        
        $text = [RdcMan.Encryption]::DecryptString($pass, $EncryptionSettings)

        return $text

    }

    [void] updatePasswordNode([array]$logonCredentials){

        foreach ($logonCredential in $logonCredentials) {

            [System.Xml.XmlElement]$element = $logonCredential.SelectSingleNode('password')

            $attr = $element.Attributes['storeAsClearText']

            if (-not $attr) {

                $text = $this.decodePassword($element.InnerText)
                $element.InnerText = $text
                $element.SetAttribute('storeAsClearText', $true)
            }
        }
    }

    # 将 rdc 文件改为明文
    [void] updateFile($file) {

        # 载入所有密码

        [XML]$xmlfile = Get-Content $file

        $this.updatePasswordNode($xmlfile.RDCMan.file.server.logonCredentials)

        $this.updatePasswordNode($xmlfile.RDCMan.file.group.server.logonCredentials)        

        $xmlfile.Save($file)
    }

    [void] writeLog($message) {
        [LogHelper]::writeLog($message, $null)
    }

    [void] writeLog($message, $exception) {
        [LogHelper]::writeLog($message, $exception)
    }

}

[LogHelper]::startLog('rdc');

$rdcFile = [RdcFile]::new()

# $rdgFile = "C:\Users\icoms\Desktop\我的远程.rdg"
$rdgFile = "C:\Users\icoms\Desktop\lightyear.rdg"
$rdcFile.updateFile($rdgFile)

$rdcFile.Dispose()

[LogHelper]::endLog();